@@ -68,8 +68,10 @@ extern SV * PerlXlib_set_magic_dpy(SV *sv, Display *dpy) {
         if (*fp && SvROK(*fp) && SvRV(*fp) != SvRV(sv))
             warn("Replacing cached connection object for Display* 0x%p!", dpy);
         // New weak-ref to this object
-        if (!*fp) *fp= newRV(SvRV(sv));
-        else sv_setsv(*fp, sv);
+        // Docs warn that sv_setsv might de-allocate mortal sources, so inc ref count temporarily
+        SvREFCNT_inc(sv);
+        if (!*fp) *fp= newSVsv(sv); else sv_setsv(*fp, sv);
+        sv_2mortal(sv);
         sv_rvweaken(*fp);
     }
     return sv;
@@ -78,25 +80,28 @@ extern SV * PerlXlib_set_magic_dpy(SV *sv, Display *dpy) {
 // Converting a Display* to a \X11::Xlib is difficult because we want
 // to re-use the existing object.  We cache them in %X11::Xlib::_connections.
 // This function returns a mortal strong-reference to an instance of X11::Xlib (or subclass)
-extern SV * PerlXlib_obj_for_display(Display *dpy) {
+extern SV * PerlXlib_obj_for_display(Display *dpy, int create) {
     SV **fp, *self;
     if (!dpy) {
         // Translate NULL to undef
         return &PL_sv_undef;
     }
     else {
-        fp= hv_fetch(get_hv("X11::Xlib::_connections", GV_ADD), (void*) &dpy, sizeof(dpy), 1);
+        fp= hv_fetch(get_hv("X11::Xlib::_connections", GV_ADD), (void*) &dpy, sizeof(dpy), 0);
         // Return existing object if we have one for this Display* already
-        if (!fp) croak("failed to add item to hash (tied?)");
-        if (*fp && SvROK(*fp)) {
+        if (fp && *fp && SvROK(*fp)) {
             // create strong-ref from weakref
-            return sv_2mortal(newRV(SvRV(*fp)));
+            return sv_mortalcopy(*fp);
         }
-        else {
+        else if (create) {
             // Always create instance of X11::Xlib.  X11::Xlib::Display can re-bless as needed.
             self= sv_2mortal(newRV_noinc((SV*) newHV()));
             sv_bless(self, gv_stashpv("X11::Xlib", GV_ADD));
-            return PerlXlib_set_magic_dpy(self, dpy);
+            PerlXlib_set_magic_dpy(self, dpy); // This also adds it to the _connections cache
+            return self;
+        }
+        else {
+            return sv_2mortal(newSVuv(PTR2UV(dpy)));
         }
     }
 }
@@ -128,44 +133,70 @@ XID PerlXlib_sv_to_xid(SV *sv) {
 #ifndef X11_Xlib_Struct_Padding
 #define X11_Xlib_Struct_Padding 64
 #endif
-void* PerlXlib_get_struct_ptr(SV *sv, const char* pkg, int struct_size, PerlXlib_struct_pack_fn *packer) {
-    SV *tmp;
+// Coercions allowed for RValue:
+//   foo( "buffer_of_the_correct_length_or_more" );
+//   foo( \"ref_to_buffer_of_the_correct_length_or_more" );
+//   foo( \%hashref_of_fields );
+//   foo( bless(\"buffer_of_correct_length_or_more", "pkg_or_subclass") )
+// Coercions allowed for LValue:
+//   foo( my $x= undef );
+//   foo( "buffer_of_correct_length_or_more" );
+//   foo( \(my $x= undef) );
+//   foo( \"buffer_of_correct_length_or_more" );
+//   foo( bless(\"buffer_of_correct_length_or_more", "any_struct_class") )
+void* PerlXlib_get_struct_ptr(SV *sv, int lvalue, const char* pkg, int struct_size, PerlXlib_struct_pack_fn *packer) {
+    SV *tmp, *ref= NULL;
     void* buf;
+    size_t n;
 
     if (SvROK(sv)) {
+        ref= sv;
+        sv= SvRV(sv);
         // Follow scalar refs, to get to the buffer of a blessed object
-        if (SvTYPE(SvRV(sv)) == SVt_PVMG)
-            sv= SvRV(sv);
-            // continue below using this SV
-
+        if (SvTYPE(sv) == SVt_PVMG) {
+            // If it is a blessed object, ensure the type matches
+            if (sv_isobject(ref) && !sv_isa(ref, pkg)) {
+                if (!sv_derived_from(ref, lvalue? "X11::Xlib::Struct" : pkg)) {
+                    buf= SvPV(ref, n);
+                    croak("Can't coerce %.*s to %s %s", n, buf, pkg, lvalue? "lvalue":"rvalue");
+                }
+            }
+        }
         // Also accept a hashref, which we pass to "pack"
-        else if (SvTYPE(SvRV(sv)) == SVt_PVHV) {
+        else if (SvTYPE(sv) == SVt_PVHV) {
+            if (lvalue) croak("Can't coerce hashref to %s lvalue", pkg);
             // Need a buffer that lasts for the rest of our XS call stack.
             // Cheat by using a mortal SV :-)
             tmp= sv_2mortal(newSV(struct_size + X11_Xlib_Struct_Padding));
             buf= SvPVX(tmp);
-            packer(buf, (HV*) SvRV(sv), 0);
+            packer(buf, (HV*) sv, 0);
             return buf;
         }
-    }
-    // If uninitialized, initialize to a blessed struct object
-    else if (!SvOK(sv)) {
-        sv= newSVrv(sv, pkg);
-        // sv is now the referenced scalar, which is undef, and gets inflated next
+        else if (SvTYPE(sv) >= SVt_PVAV) { // not a scalar
+            buf= SvPV(ref, n);
+            croak("Can't coerce %.*s to %s %s", n, buf, pkg, lvalue? "lvalue":"rvalue");
+        }
     }
     
+    // If uninitialized, initialize to a blessed struct object,
+    //  unless we're looking at \undef in which case just initialize to a string
     if (!SvOK(sv)) {
+        if (!lvalue) croak("Can't coerce %sundef to %s rvalue", ref? "\\" : "", pkg);
+        if (!ref) {
+            ref= sv, sv= newSVrv(sv, pkg);
+            // sv is now the referenced scalar, which is undef, and gets inflated next
+        }
         sv_setpvn(sv, "", 0);
         SvGROW(sv, struct_size+X11_Xlib_Struct_Padding);
         SvCUR_set(sv, struct_size);
         memset(SvPVX(sv), 0, struct_size+1);
     }
     else if (!SvPOK(sv))
-        croak("Paramters requiring %s can only be coerced from scalar, scalar ref, hashref, or undef", pkg);
+        croak("Paramters requiring %s can only be coerced from string, string ref, hashref, or undef", pkg);
     else if (SvCUR(sv) < struct_size)
-        croak("Scalars used as %s must be at least %d bytes long (got %d)",
-            pkg, struct_size, SvCUR(sv));
+        croak("Scalars used as %s must be at least length %d (got %d)", pkg, struct_size, SvCUR(sv));
     // Make sure we have the padding even if the user tinkered with the buffer
+    SvPV_force(sv, n);
     SvGROW(sv, struct_size+X11_Xlib_Struct_Padding);
     return SvPVX(sv);
 }
@@ -176,7 +207,7 @@ int PerlXlib_X_error_handler(Display *d, XErrorEvent *e) {
     SAVETMPS;
     PUSHMARK(SP);
     EXTEND(SP, 1);
-    PUSHs(sv_2mortal(sv_setref_pvn(newSV(0), "X11::Xlib::XEvent::XErrorEvent", (void*) e, sizeof(XEvent))));
+    PUSHs(sv_2mortal(sv_setref_pvn(newSV(0), "X11::Xlib::XErrorEvent", (void*) e, sizeof(XEvent))));
     PUTBACK;
     call_pv("X11::Xlib::_error_nonfatal", G_VOID|G_DISCARD|G_EVAL|G_KEEPERR);
     FREETMPS;
@@ -206,7 +237,7 @@ int PerlXlib_X_IO_error_handler(Display *d) {
     SAVETMPS;
     PUSHMARK(SP);
     EXTEND(SP, 1);
-    PUSHs(PerlXlib_obj_for_display(d));
+    PUSHs(PerlXlib_obj_for_display(d, 1));
     PUTBACK;
     call_pv("X11::Xlib::_error_fatal", G_VOID|G_DISCARD|G_EVAL|G_KEEPERR);
     FREETMPS;
@@ -236,40 +267,40 @@ void PerlXlib_install_error_handlers(Bool nonfatal, Bool fatal) {
 
 const char* PerlXlib_xevent_pkg_for_type(int type) {
   switch (type) {
-  case ButtonPress: return "X11::Xlib::XEvent::XButtonEvent";
-  case ButtonRelease: return "X11::Xlib::XEvent::XButtonEvent";
-  case CirculateNotify: return "X11::Xlib::XEvent::XCirculateEvent";
-  case CirculateRequest: return "X11::Xlib::XEvent::XCirculateRequestEvent";
-  case ClientMessage: return "X11::Xlib::XEvent::XClientMessageEvent";
-  case ColormapNotify: return "X11::Xlib::XEvent::XColormapEvent";
-  case ConfigureNotify: return "X11::Xlib::XEvent::XConfigureEvent";
-  case ConfigureRequest: return "X11::Xlib::XEvent::XConfigureRequestEvent";
-  case CreateNotify: return "X11::Xlib::XEvent::XCreateWindowEvent";
-  case DestroyNotify: return "X11::Xlib::XEvent::XDestroyWindowEvent";
-  case EnterNotify: return "X11::Xlib::XEvent::XCrossingEvent";
-  case Expose: return "X11::Xlib::XEvent::XExposeEvent";
-  case FocusIn: return "X11::Xlib::XEvent::XFocusChangeEvent";
-  case FocusOut: return "X11::Xlib::XEvent::XFocusChangeEvent";
-  case GenericEvent: return "X11::Xlib::XEvent::XGenericEvent";
-  case GraphicsExpose: return "X11::Xlib::XEvent::XGraphicsExposeEvent";
-  case GravityNotify: return "X11::Xlib::XEvent::XGravityEvent";
-  case KeyPress: return "X11::Xlib::XEvent::XKeyEvent";
-  case KeyRelease: return "X11::Xlib::XEvent::XKeyEvent";
-  case KeymapNotify: return "X11::Xlib::XEvent::XKeymapEvent";
-  case LeaveNotify: return "X11::Xlib::XEvent::XCrossingEvent";
-  case MapNotify: return "X11::Xlib::XEvent::XMapEvent";
-  case MapRequest: return "X11::Xlib::XEvent::XMapRequestEvent";
-  case MappingNotify: return "X11::Xlib::XEvent::XMappingEvent";
-  case MotionNotify: return "X11::Xlib::XEvent::XMotionEvent";
-  case NoExpose: return "X11::Xlib::XEvent::XNoExposeEvent";
-  case PropertyNotify: return "X11::Xlib::XEvent::XPropertyEvent";
-  case ReparentNotify: return "X11::Xlib::XEvent::XReparentEvent";
-  case ResizeRequest: return "X11::Xlib::XEvent::XResizeRequestEvent";
-  case SelectionClear: return "X11::Xlib::XEvent::XSelectionClearEvent";
-  case SelectionNotify: return "X11::Xlib::XEvent::XSelectionEvent";
-  case SelectionRequest: return "X11::Xlib::XEvent::XSelectionRequestEvent";
-  case UnmapNotify: return "X11::Xlib::XEvent::XUnmapEvent";
-  case VisibilityNotify: return "X11::Xlib::XEvent::XVisibilityEvent";
+  case ButtonPress: return "X11::Xlib::XButtonEvent";
+  case ButtonRelease: return "X11::Xlib::XButtonEvent";
+  case CirculateNotify: return "X11::Xlib::XCirculateEvent";
+  case CirculateRequest: return "X11::Xlib::XCirculateRequestEvent";
+  case ClientMessage: return "X11::Xlib::XClientMessageEvent";
+  case ColormapNotify: return "X11::Xlib::XColormapEvent";
+  case ConfigureNotify: return "X11::Xlib::XConfigureEvent";
+  case ConfigureRequest: return "X11::Xlib::XConfigureRequestEvent";
+  case CreateNotify: return "X11::Xlib::XCreateWindowEvent";
+  case DestroyNotify: return "X11::Xlib::XDestroyWindowEvent";
+  case EnterNotify: return "X11::Xlib::XCrossingEvent";
+  case Expose: return "X11::Xlib::XExposeEvent";
+  case FocusIn: return "X11::Xlib::XFocusChangeEvent";
+  case FocusOut: return "X11::Xlib::XFocusChangeEvent";
+  case GenericEvent: return "X11::Xlib::XGenericEvent";
+  case GraphicsExpose: return "X11::Xlib::XGraphicsExposeEvent";
+  case GravityNotify: return "X11::Xlib::XGravityEvent";
+  case KeyPress: return "X11::Xlib::XKeyEvent";
+  case KeyRelease: return "X11::Xlib::XKeyEvent";
+  case KeymapNotify: return "X11::Xlib::XKeymapEvent";
+  case LeaveNotify: return "X11::Xlib::XCrossingEvent";
+  case MapNotify: return "X11::Xlib::XMapEvent";
+  case MapRequest: return "X11::Xlib::XMapRequestEvent";
+  case MappingNotify: return "X11::Xlib::XMappingEvent";
+  case MotionNotify: return "X11::Xlib::XMotionEvent";
+  case NoExpose: return "X11::Xlib::XNoExposeEvent";
+  case PropertyNotify: return "X11::Xlib::XPropertyEvent";
+  case ReparentNotify: return "X11::Xlib::XReparentEvent";
+  case ResizeRequest: return "X11::Xlib::XResizeRequestEvent";
+  case SelectionClear: return "X11::Xlib::XSelectionClearEvent";
+  case SelectionNotify: return "X11::Xlib::XSelectionEvent";
+  case SelectionRequest: return "X11::Xlib::XSelectionRequestEvent";
+  case UnmapNotify: return "X11::Xlib::XUnmapEvent";
+  case VisibilityNotify: return "X11::Xlib::XVisibilityEvent";
   default: return "X11::Xlib::XEvent";
   }
 }
@@ -642,7 +673,7 @@ void PerlXlib_XEvent_pack(XEvent *s, HV *fields, Bool consume) {
       if (fp && *fp) { s->xvisibility.state= SvIV(*fp);; if (consume) hv_delete(fields, "state", 5, G_DISCARD); }
       break;
     default:
-      croak("Unknown XEvent type %d", s->type);
+      warn("Unknown XEvent type %d", s->type);
     }
 }
 
@@ -651,7 +682,7 @@ void PerlXlib_XEvent_unpack(XEvent *s, HV *fields) {
     // If it does, we need to clean up the value!
     SV *sv= NULL;
     if (!hv_store(fields, "type", 4, (sv= newSViv(s->type)), 0)) goto store_fail;
-    if (!hv_store(fields, "display"   ,  7, (sv=PerlXlib_obj_for_display(s->xany.display)), 0)) goto store_fail;
+    if (!hv_store(fields, "display"   ,  7, (sv=SvREFCNT_inc(PerlXlib_obj_for_display(s->xany.display, 0))), 0)) goto store_fail;
     if (!hv_store(fields, "send_event", 10, (sv=newSViv(s->xany.send_event)), 0)) goto store_fail;
     if (!hv_store(fields, "serial"    ,  6, (sv=newSVuv(s->xany.serial)), 0)) goto store_fail;
     if (!hv_store(fields, "type"      ,  4, (sv=newSViv(s->xany.type)), 0)) goto store_fail;
@@ -1015,7 +1046,7 @@ void PerlXlib_XSizeHints_pack(XSizeHints *s, HV *fields, Bool consume) {
     if (fp && *fp) { s->flags |= PBaseSize; s->base_width= SvIV(*fp); if (consume) hv_delete(fields, "base_width", 10, G_DISCARD); }
 
     fp= hv_fetch(fields, "flags", 5, 0);
-    if (fp && *fp) {  s->flags= SvIV(*fp); if (consume) hv_delete(fields, "flags", 5, G_DISCARD); }
+    if (fp && *fp) { s->flags= SvIV(*fp); if (consume) hv_delete(fields, "flags", 5, G_DISCARD); }
 
     fp= hv_fetch(fields, "height", 6, 0);
     if (fp && *fp) { s->flags |= PSize; s->height= SvIV(*fp); if (consume) hv_delete(fields, "height", 6, G_DISCARD); }
