@@ -8,7 +8,7 @@ use base qw(Exporter DynaLoader);
 use Carp;
 use Try::Tiny;
 
-our $VERSION = '0.09_03';
+our $VERSION = '0.09_04';
 
 sub dl_load_flags { 1 } # Make PerlXLib.c functions available to other XS modules
 
@@ -29,6 +29,17 @@ my %_constants= (
     KeymapNotify LeaveNotify MapNotify MappingNotify MotionNotify NoExpose
     PropertyNotify ReparentNotify ResizeRequest SelectionClear SelectionNotify
     SelectionRequest UnmapNotify VisibilityNotify )],
+  const_event_input_state => [qw( Button1Mask Button2Mask Button3Mask
+    Button4Mask Button5Mask ControlMask LockMask Mod1Mask Mod2Mask Mod3Mask
+    Mod4Mask Mod5Mask ShiftMask )],
+  const_event_mask => [qw( Button1MotionMask Button2MotionMask
+    Button3MotionMask Button4MotionMask Button5MotionMask ButtonMotionMask
+    ButtonPressMask ButtonReleaseMask ColormapChangeMask EnterWindowMask
+    ExposureMask FocusChangeMask KeyPressMask KeyReleaseMask KeymapStateMask
+    LeaveWindowMask NoEventMask OwnerGrabButtonMask PointerMotionHintMask
+    PointerMotionMask PropertyChangeMask ResizeRedirectMask
+    StructureNotifyMask SubstructureNotifyMask SubstructureRedirectMask
+    VisibilityChangeMask )],
   const_sizehint => [qw( PAspect PBaseSize PMaxSize PMinSize PPosition
     PResizeInc PSize PWinGravity USPosition USSize )],
   const_visual => [qw( VisualAllMask VisualBitsPerRGBMask VisualBlueMaskMask
@@ -48,9 +59,13 @@ my %_functions= (
   fn_event => [qw( XCheckMaskEvent XCheckTypedEvent XCheckTypedWindowEvent
     XCheckWindowEvent XFlush XNextEvent XPutBackEvent XSelectInput XSendEvent
     XSync )],
-  fn_key => [qw( IsFunctionKey IsKeypadKey IsMiscFunctionKey IsModifierKey
-    IsPFKey IsPrivateKeypadKey XGetKeyboardMapping XKeysymToKeycode
-    XKeysymToString XStringToKeysym )],
+  fn_input => [qw(  )],
+  fn_keymap => [qw( XDisplayKeycodes XGetKeyboardMapping XGetModifierMapping
+    XKeysymToKeycode XLookupString XRefreshKeyboardMapping XSetModifierMapping
+    )],
+  fn_keysym => [qw( IsFunctionKey IsKeypadKey IsMiscFunctionKey IsModifierKey
+    IsPFKey IsPrivateKeypadKey XConvertCase XKeysymToString XStringToKeysym
+    )],
   fn_pix => [qw( XCreateBitmapFromData XCreatePixmap
     XCreatePixmapFromBitmapData XFreePixmap )],
   fn_screen => [qw( DefaultColormap DefaultDepth DefaultGC DefaultScreen
@@ -73,7 +88,7 @@ our %EXPORT_TAGS= (
     functions => [ map { @$_ } values %_functions ],
     all => \@EXPORT_OK,
 );
-our @EXPORT= @{ $EXPORT_TAGS{fn_key} };
+our @EXPORT= @{ $EXPORT_TAGS{fn_keysym} };
 
 # Used by XS.  In the spirit of letting perl users violate encapsulation
 #  as needed, the XS code exposes its globals to Perl.
@@ -179,8 +194,8 @@ X11::Xlib - Low-level access to the X11 library
   
   use X11::Xlib;
   my $display= X11::Xlib->new($conn_string);  # shortcut for X11::Xlib::Display->new
-  $display->fake_motion(undef, 50, 50)
-  $display->flush;
+  my $window= $display->new_window({ x => 0, y => 0, width => 50, height => 50);
+  $window->show();
 
 =head1 DESCRIPTION
 
@@ -198,6 +213,8 @@ on that, you get a more friendly wrapper around Xlib that helps you manage
 resource lifespan, wraps XIDs with perl objects, and does some sanity checking
 on the state of the library when you call methods.
 
+=cut
+
 =head1 ATTRIBUTES
 
 The X11::Xlib connection is a hashref with a few attributes and methods
@@ -205,7 +222,7 @@ independent of Xlib.
 
 =head2 autoclose
 
-Boolean flag that determines whether the destructor will call L</XCloseDisplay>.
+Boolean flag that determines whether the destructor will call XCloseDisplay.
 Defaults to true for connections returned by L</XOpenDisplay>.
 
 =head1 FUNCTIONS
@@ -234,8 +251,8 @@ handler returns Xlib still kills your program.  Under normal circumstances you
 would have to perform all cleanup with your stack tied up through Xlib, but
 this library cheats by using croak (C<longjmp>) to escape the callback and let
 you wrap up your script in a normal manner.  B<However>, after a fatal
-error Xlib's internal state could be dammaged, so it is unsafe to make any more
-Xlib calls.  This library tries to help enforce that by invalidating all the
+error Xlib's internal state could be damaged, so it is unsafe to make any more
+Xlib calls.  The library tries to help assert this by invalidating all the
 connection objects.
 
 If you really need your program to keep running your best bet is to state-dump
@@ -297,17 +314,17 @@ Determines what resources are freed upon disconnect.  See X11 documentation.
 =head2 COMMUNICATION FUNCTIONS
 
 Most of these functions return an L</XEvent> by way of an "out" parameter that
-gets overwritten during the call, in the style of C.  You may pass in an
-undefined scalar to be automatically allocated for you.
+gets overwritten during the call, in the style of C.  The variable receiving the
+event does not need to be initialized.
 
 =head3 XNextEvent
 
   XNextEvent($display, my $event_return)
   ... # event scalar is populated with event
 
-You probably don't want this.  It blocks forever until an event arrives.
-I added it for completeness.  See L<X11::Xlib::Display/wait_event>
-for a more Perl-ish interface.
+You probably don't want this.  It blocks forever until an event arrives, even
+ignoring signals.  I added it for completeness.
+See L<X11::Xlib::Display/wait_event> for a more perl-ish interface.
 
 =head3 XCheckMaskEvent
 
@@ -336,21 +353,23 @@ blocking.
   XSendEvent($display, $window, $propagate, $event_mask, $xevent)
     or die "Xlib hates us";
 
-Send an XEvent.
+Send an XEvent to the server, to be redispatched however appropriate.
 
 =head3 XPutBackEvent
 
   XPutBackEvent($display, $xevent)
 
-Push an XEvent back onto the queue.  This can also presumably put an
-arbitrarily bogus event onto your own queue since it returns void.
+Push an XEvent back onto your own incoming queue.
+This can presumably put arbitrarily bogus events onto your own queue
+since it returns void.
 
 =head3 XFlush
 
   XFlush($display)
 
-Push any queued messages to the X11 server.  If you're wondering why nothing
-happened when you called an XTest function, this is why.
+Push any queued messages to the X11 server.  Some Xlib calls perform an
+implied flush of the queue, while others don't.  If you're wondering why
+nothing happened when you called an XTest function, this is why.
 
 =head3 XSync
 
@@ -454,9 +473,9 @@ Returns true if it found a matching visual.
   my @info_structs= XGetVisualInfo($display, $mask, $xvis_template);
 
 Returns a list of L</XVisualInfo> each describing an available L</Visual>
-which matches the template you provided. (also an C<XVisualInfo>)
+which matches the template you provided. (which is also an C<XVisualInfo>)
 
-C<$mask> can be one of:
+C<$mask> can be any combination of:
 
   VisualIDMask
   VisualScreenMask
@@ -599,7 +618,7 @@ It is initially unmapped.  See L</XMapWindow>.
 
   XMapWindow($display, $window);
 
-Ask the X server to show a window.  This call asynchronous and you should call
+Ask the X server to show a window.  This call is asynchronous and you should call
 L</XFlush> if you want it to appear immediately.  The window will only appear if
 the parent window is also mapped.  The server sends back a MapNotify event if
 the Window event mask allows it, and if a variety of other conditions are met.
@@ -661,7 +680,7 @@ happen immediately.
 
   XTestFakeMotionEvent($display, $screen, $x, $y, $EventSendDelay)
 
-Fake a mouse movement on screen number C<$screen> to position C<$x>,C<$y>.
+Fake a mouse movement to position C<$x>,C<$y> on screen number C<$screen>.
 
 The optional C<$EventSendDelay> parameter specifies the number of milliseconds to wait
 before sending the event. The default is 10 milliseconds.
@@ -686,11 +705,7 @@ and C<$pressed> indicates if the key was pressed or released.
 The optional C<$EventSendDelay> parameter specifies the number of milliseconds to wait
 before sending the event. The default is 10 milliseconds.
 
-=head3 XBell
-
-  XBell($display, $percent)
-
-Make the X server emit a sound.
+=head2 KEYCODE FUNCTIONS
 
 =head3 XQueryKeymap
 
@@ -698,7 +713,58 @@ Make the X server emit a sound.
 
 Return a list of the key codes currently pressed on the keyboard.
 
-=head2 KEYCODE FUNCTIONS
+=head3 XGetKeyboardMapping
+
+  XGetKeyboardMapping($display, $keycode, $count)
+
+Return an array of KeySym numbers corresponding to C<$count> keys, starting
+at C<$keycode>.
+
+Each value in the array corresponds to a combination of key modifiers (Shift,
+Lock, Mode).  The X11 server may return a variable number of codes per key,
+which you can determine by dividing the total number of values returned by this
+function by the C<$count>.
+
+For an easier interface, see L<X11::Xlib::Keymap>.
+
+=head3 XGetModifierMapping
+
+  my $mapping= XGetModifierMapping($display);
+
+Return an arrayref of 8 arrayrefs, one for each of the possible modifiers.
+The inner arrayrefs can contain a variable number of key codes which belong
+to the modifier group.  See L<X11::Xlib::Keymap> for an explanation.
+
+=head3 XSetModifierMapping
+
+  XSetModifierMapping($display, \@modifiers);
+
+C<@modifiers> is an array of 8 arrayrefs, each holding the set of key codes
+that are part of the modifier.  This is the same format as returned by
+C<XGetModifierMapping>.
+
+=head3 XRefreshKeyboardMapping
+
+  XRefreshKeyboardMapping($mapping_event);
+
+Given a XMappingEvent, reload the internal Xlib cache for the parts of the
+keymap or modmap which have changed.
+
+The functions below (using the internal Xlib cache) are an alternative to
+processing the keymap directly.
+
+=head3 XLookupString
+
+  XLookupString($key_event, $text_out, $keycode_out);
+
+Given a XKeyEvent, translate the key code and modifiers vs. the internal Xlib
+cached keymap/modmap and write the text name of the key into C<$text_out>.
+This will either be a name of a key, or the character the key normally
+generates in a Latin-1 environment.  If C<$keycode_out> is given, it will be
+overwritten with the numeric value of the KeySym.
+
+(If you want to do more than Latin-1, see L<X11::Xlib::Keymap> for utilities
+ to manipulate the keymap directly.)
 
 =head3 XKeysymToKeycode
 
@@ -706,17 +772,6 @@ Return a list of the key codes currently pressed on the keyboard.
 
 Return the key code corresponding to the character number C<$keysym>.
 
-=head3 XGetKeyboardMapping
-
-  XGetKeyboardMapping($display, $keycode, $count)
-
-Return an array of character numbers corresponding to the key C<$keycode>.
-
-Each value in the array corresponds to the action of a key modifier (Shift, Alt).
-
-C<$count> is the number of the keycode to return. The default value is 1, e.g.
-it returns the character corresponding to the given $keycode.
-
 =head3 XKeysymToString
 
   XKeysymToString($keysym)
@@ -733,6 +788,12 @@ Return the keysym number for the human-readable character C<$string>.
 
 C<XStringToKeysym> is the exact reverse of C<XKeysymToString>.
 
+=head3 XConvertCase
+
+  XConvertCase($keysym, $lowercase_out, $uppercase_out);
+
+Return the lowercase and uppercase KeySyms values for C<$keysym>.
+
 =head3 IsFunctionKey
 
   IsFunctionKey($keysym)
@@ -769,6 +830,12 @@ Xlib docs are fun.  No mention of what "PF" might be.
 
 True for vendor-private key codes.
 
+=head3 XBell
+
+  XBell($display, $percent)
+
+Make the X server emit a sound.
+
 =head1 STRUCTURES
 
 Xlib has a lot of C B<struct>s.  Most of them do not have much "depth"
@@ -809,7 +876,8 @@ perl-ish interface and some helper methods to "DWIM".
 The Xlib C<Screen*> is not exported by this module, since most methods that
 use a C<Screen*> have a matching method that uses a C<Display*>.
 If you are using the object-oriented L<Display|X11::Xlib::Display> you then
-get L<Screen|X11::Xlib::Screen> objects for convenience.
+get L<Screen|X11::Xlib::Screen> objects for convenience, but they are just
+a wrapper around the Display and screen number instead of screen pointer.
 
 =head2 Visual
 
@@ -874,11 +942,15 @@ This module provides a higher-level API for X input simulation and testing.
 Functions provided by X11/Xlib are mostly included in the L<Gtk2> binding, but
 through the GTK API and perl objects.
 
+=item L<X11::Protocol>
+
+Pure-perl implementation of the X11 protocol.
+
 =back
 
 =head1 TODO
 
-This module still only covers a fraction of the Xlib API.
+This module still only covers a small fraction of the Xlib API.
 Patches are welcome :)
 
 =head1 AUTHOR
