@@ -27,8 +27,7 @@ fields names.
 sub new {
     my $class= shift;
     $class= ref $class if ref $class;
-    my $buffer;
-    my $self= bless \$buffer, $class;
+    my $self= bless \(my $buffer), $class;
     $self->_initialize;
     $self->apply(@_) if @_; # If arguments, then initialize using apply
     $self;
@@ -48,9 +47,9 @@ sub initialize {
 
   $struct->pack( \%fields, $consume, $warn );
 
-Pack field values into the bytes of the struct.  Only C<\%fields> is required.
+Pack field values into the bytes of the struct.  Only C<%fields> is required.
 
-If C<$consume> is true, then remove any key of \%fields that was processed.
+If C<$consume> is true, then remove any key of C<%fields> that was processed.
 
 If C<$warn> is true, then emit a warning if any un-recognized field was given.
 
